<?php

declare(strict_types=1);

namespace mirzaev\vk\api\methods;

// Файлы проекта
use mirzaev\vk\robots\user;

// Встроенные библиотеки
use Exception;

/**
 * Аккаунт
 *
 * @method public getInfo(?array $fields = null) Запросить информацию об аккаунте
 * @method public getProfileInfo(?array $fields = null) Запросить информацию о профиле
 *
 * @see https://dev.vk.com/method/account
 *
 * @package mirzaev\vk\api
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class account extends method
{
    /**
     * Конструктор
     *
     * @param user $user Робот
     */
    public function __construct(
        protected user $robot
    ) {
    }

    /**
     * Запросить информацию об аккаунте
     *
     * @see https://dev.vk.com/method/account.getInfo
     *
     * @param array $fields Выбор полей с запрашиваемой информацией (оставить пустым, если нужны все)
     *
     * @return ?array Информация об аккаунте, если получена
     *
     * @todo
     * 1. Доделать
     * 2. Написать обработчик ошибок возвращаемых ВКонтакте
     */
    public function getInfo(array $fields = []): array
    {
        // Реиницилазиция
        $this->robot->api->reinit();

        foreach ($fields as $key => $field) {
            // Перебор запрашиваемых полей с информацией

            // Запись запрашиваемого поля
            $this->robot->api['fields'] .= $field;

            // Запись разделителя
            if ($key === array_key_last($fields)) break;
            else $this->robot->api['fields'] .= ', ';
        }

        // Запрос
        $request = json_decode($this->robot->browser->request('POST', 'account.getInfo', ['form_params' => $this->robot->api->settings])->getBody()->getContents());

        // Если в ответе ошибка
        if (isset($request->error)) {
            throw new Exception('ВКонтакте: ' . $request->error->error_msg, $request->error->error_code);
        }

        return $request->response;
    }

    /**
     * Запросить информацию о профиле
     *
     * @see https://dev.vk.com/method/account.getProfileInfo
     *
     * @param array $fields Выбор полей с запрашиваемой информацией (оставить пустым, если нужны все)
     *
     * @return ?array Информация об аккаунте, если получена
     *
     * @todo
     * 1. Доделать
     * 2. Написать обработчик ошибок возвращаемых ВКонтакте
     */
    public function getProfileInfo(array $fields = []): array
    {
        // Реиницилазиция
        $this->robot->api->reinit();

        foreach ($fields as $key => $field) {
            // Перебор запрашиваемых полей с информацией

            // Запись запрашиваемого поля
            $this->robot->api['fields'] .= $field;

            // Запись разделителя
            if ($key === array_key_last($fields)) break;
            else $this->robot->api['fields'] .= ', ';
        }

        // Запрос
        $request = json_decode($this->robot->browser->request('POST', 'account.getProfileInfo', ['form_params' => $this->robot->api->settings])->getBody()->getContents());

        // Если в ответе ошибка
        if (isset($request->error)) {
            throw new Exception('ВКонтакте: ' . $request->error->error_msg, $request->error->error_code);
        }

        return $request->response;
    }
}
