<?php

declare(strict_types=1);

namespace mirzaev\vk\api\methods;

// Файлы проекта
use mirzaev\vk\robots\user;

// Встроенные библиотеки
use Exception;

/**
 * Пользователь
 *
 *
 * @see https://dev.vk.com/method/users
 *
 * @package mirzaev\vk\api
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class users extends method
{
    /**
     * Конструктор
     *
     * @param user $user Робот
     */
    public function __construct(
        protected user $robot
    ) {
    }

    /**
     * Запросить информацию о пользователе
     *
     * @see https://dev.vk.com/method/users.get
     *
     * @param array $user_ids Выбор пользователей для запроса информации
     * @param array $fields Выбор дополнительных запрашиваемых полей
     * @param string $name_case Падеж
     *
     * @return ?array Информация об аккаунте, если получена
     *
     * @todo
     * 1. Доделать
     * 2. Написать обработчик ошибок возвращаемых ВКонтакте
     */
    public function get(array $user_ids = [], array $fields = [], string $name_case = ''): array
    {
        // Реиницилазиция
        $this->robot->api->reinit();

        // Инициализация пользователей для запроса информации
        if (!empty($user_ids)) $this->robot->api['user_ids'] = '';

        foreach ($user_ids as $key => $user_id) {
            // Перебор пользователей для получения информации

            // Запись пользователя
            $this->robot->api['user_ids'] .= $user_id;

            // Запись разделителя
            if ($key === array_key_last($user_ids)) break;
            else $this->robot->api['user_ids'] .= ', ';
        }

        // Инициализация дополнительных запрашиваемых полей
        if (!empty($fields)) $this->robot->api['fields'] = '';

        foreach ($fields as $key => $field) {
            // Перебор дополнительных запрашиваемых полей

            // Запись запрашиваемого дополнительного поля
            $this->robot->api['fields'] .= $field;

            // Запись разделителя
            if ($key === array_key_last($fields)) break;
            else $this->robot->api['fields'] .= ', ';
        }

        // Инициализация падежа
        if (!empty($name_case)) $this->robot->api['name_case'] = $name_case;

        // Запрос
        $request = json_decode($this->robot->browser->request('POST', 'users.get', ['form_params' => $this->robot->api->settings])->getBody()->getContents());

        // Если в ответе ошибка
        if (isset($request->error)) {
            throw new Exception('ВКонтакте: ' . $request->error->error_msg, $request->error->error_code);
        }

        return $request->response;
    }
}
