<?php

declare(strict_types=1);

namespace mirzaev\vk\robots;

use Throwable;
use Exception;

use mirzaev\vk\robots\robot,
    mirzaev\vk\api\settings as api,
    mirzaev\vk\api\longpoll;

/**
 * Робот-группа
 *
 * @property longpoll $longpoll LongPoll-сессия
 *
 * @method public function __construct(int|null $id, string|null $key) Конструктор
 * @method public function __set($name, $value) Запись свойства
 * @method public function __get($name) Чтение свойства
 * @method public function __isset($name) Проверка на инициализированность свойства
 *
 * @package mirzaev\vk\robots
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class group extends robot
{
    /**
     * @var longpoll $longpoll LongPoll-сессия
     */
    protected longpoll $longpoll;

    /**
     * Конструктор
     *
     * @param int|null $id Идентификатор
     * @param string|null $key Ключ
     */
    public function __construct(
        protected ?int $id = null,
        protected ?string $key = null
    ) {
        parent::__construct($id, $key);

        // Инициализация данных API
        $this->api['group_id'] = $this->id;
    }

    /**
     * Запись свойства
     *
     * @param string $name Название
     * @param mixed $value Значение
     *
     * @see mirzaev\vk\robots\robot Наследуемый метод
     *
     * @return void
     */
    public function __set(string $name, mixed $value): void
    {
        try {
            parent::__set($name, $value);
        } catch (Throwable $e) {
            // Если свойство не найдено в родительском методе

            match ($name) {
                'longpoll' => $value instanceof LongPoll ? $this->longpoll = $value : $this->longpoll = new LongPoll($this),
                default => throw new Exception($e->getMessage(), $e->getCode(), $e->getPrevious())
            };
        }
    }

    /**
     * Чтение свойства
     *
     * @param string $name Название
     *
     * @see mirzaev\vk\robots\robot Наследуемый метод
     *
     * @return mixed
     */
    public function __get(string $name): mixed
    {
        try {
            return parent::__get($name);
        } catch (Throwable $e) {
            // Если свойство не найдено в родительском методе

            if ($name === 'longpoll') {
                if (!isset($this->longpoll)) {
                    $this->longpoll = new LongPoll($this);
                }
                return $this->longpoll;
            }

            throw new Exception($e->getMessage(), $e->getCode(), $e->getPrevious());
        }
    }

    /**
     * Проверка на инициализированность свойства
     *
     * @param string $name Название
     *
     * @return mixed
     */
    public function __isset(string $name): bool
    {
        try {
            return parent::__isset($name);
        } catch (Throwable $e) {
            // Если свойство не найдено в родительском методе

            if ($name === 'longpoll') {
                return isset($this->longpoll);
            }

            throw new Exception($e->getMessage(), $e->getCode(), $e->getPrevious());
        }
    }
}
