<?php

declare(strict_types=1);

namespace mirzaev\vk\traits;

/**
 * Паттерн registry
 */
trait registry
{
    /**
     * Хеш-таблица реестра
     *
     * @var array
     */
    protected static array $registry = [];

    /**
     * Запись в реестр
     *
     * @param int $key
     * @param mixed $value
     *
     * @return void
     */
    public static function set(int $key, $value): bool
    {
        if (!array_key_exists($key, self::$registry)) {
            self::$registry[$key] = $value;
            return true;
        }

        return false;
    }

    /**
     * Извлечение из реестра
     *
     * Если не отправить ключ, то вернёт все значения
     *
     * @param int $key
     *
     * @return mixed
     */
    public static function get(int $key = null)
    {
        if (isset($key) && array_key_exists($key, self::$registry)) {
            return self::$registry[$key];
        } else {
            return self::$registry;
        }
    }

    /**
     * Удалить из реестра
     *
     * @param int $key
     *
     * @return bool
     */
    public static function delete(int $key): bool
    {
        if (array_key_exists($key, self::$registry)) {
            unset(self::$registry[$key]);
            return true;
        }

        return false;
    }
}
