<?php

declare(strict_types=1);

use PHPUnit\Framework\TestCase;

use mirzaev\vk\core;
use mirzaev\vk\robots\robot;
use mirzaev\vk\tests\settings;

use mirzaev\accounts\vk as account;

/**
 * @testdox Фото
 */
final class photosTest extends TestCase
{
    use settings;

    /**
     * @var core $core Ядро фреймворка
     */
    private static core $core;

    /**
     * @var account $account Аккаунт
     */
    protected static account $account;

    /**
     * @var robot $robot Робот
     */
    private static robot $robot;

    /**
     * @testdox Инициализация аккаунта
     * @beforeClass
     */
    public function testAccountInit(): void
    {
        // Проверка входных данных
        if (empty(self::$project_id)) {
            self::markTestSkipped('Не инициализирован идентификатор проекта');
        }

        // Инициализация аккаунта
        self::$account = new account(empty(self::$id) ? rand(0, 10) : self::$id, self::$path_accounts);

        // Запись режима SSL-протокола
        self::$account->ssl = self::$ssl ?? true;

        if (empty(self::$key)) {
            // Если не указан ключ

            // Проверка входных данных
            if (empty(self::$login)) {
                self::markTestSkipped('Не инициализирован входной аккаунта');
            } else if (empty(self::$password)) {
                self::markTestSkipped('Не инициализирован пароль аккаунта');
            }

            // Деаутентификация (на всякий случай)
            self::$account->deauth();

            // Аутентификация и генерация ключа
            self::$key = self::$account->auth(self::$login, self::$password)->key(self::$project_id);
        }

        $this->assertNotNull(self::$key, 'Ошибка при инициализации ключа аккаунта');
    }

    /**
     * @testdox Деинициализация аккаунта
     * @afterClass
     */
    public static function testAccountDeinit(): void
    {
        // Инициализация аккаунта
        self::$account = new account(empty(self::$id) ? rand(0, 10) : self::$id, self::$path_accounts);

        // Деаутентификация
        self::$account->deauth();
    }

    /**
     * @testdox Инициализация робота
     * @before
     */
    public function testRobotGroupInit(): void
    {
        // Инициализация ядра
        self::$core = core::init();

        // Сохранение количества роботов
        $count = self::$core->robots;

        // Инициализация робота
        self::$robot = self::$core->group(empty(self::$group_id) ? rand(0, 10) : self::$group_id);

        // Проверка
        $this->assertEquals(self::$core->robots, $count + 1, 'Ошибка при инициализации робота');
    }

    /**
     * @testdox Деинициализация робота
     * @after
     */
    public function testRobotGroupDeinit(): void
    {
        // Очистка реестра
        self::$core->delete();

        // Проверка
        $this->assertEmpty(self::$core->get(self::$robot->id), 'Ошибка при деинициализации робота');
    }

    /**
     * @testdox Получение адреса сервера сообщений
     */
    public function testGetMessageServer(): void
    {
        // Получение адреса сервера
        $response = self::$robot->key(self::$group_key)->photo()->getMessageServer();

        // Проверка
        $this->assertIsString($response->upload_url, 'Адрес сервера не получен');

    }

    /**
     * @testdox Загрузка фото
     */
    public function testUpload(): void
    {
        // Получение адреса сервера
        $url = self::$robot->key(self::$group_key)->photo()->getMessageServer()->upload_url;

        // Загрузка фото
        $response = self::$robot->photo()->upload(fopen(self::$path_photo, 'r'), $url);

        // Проверка
        $this->assertIsString($response->hash, 'Фото не загружено');
    }

    /**
     * @testdox Получение id фото для сообщения
     */
    public function testGetPhoto(): void
    {
        // Проверка
        self::$robot->key(self::$group_key)->photo()->getPhoto(self::$path_photo);
    }
}
